"""Headers for metadata."""

from .datatime import HEADER

START_DATETIME = f"Start Time ({HEADER})"
END_DATETIME = f"End Time ({HEADER})"
DURATION = "Duration (seconds)"
SETPOINT = "Oven Setpoint (degrees C)"
